/*******************************************************************************
 *     ___                  _   ____  ____
 *    / _ \ _   _  ___  ___| |_|  _ \| __ )
 *   | | | | | | |/ _ \/ __| __| | | |  _ \
 *   | |_| | |_| |  __/\__ \ |_| |_| | |_) |
 *    \__\_\\__,_|\___||___/\__|____/|____/
 *
 *  Copyright (c) 2014-2019 Appsicle
 *  Copyright (c) 2019-2020 QuestDB
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 ******************************************************************************/

package io.questdb.griffin.engine.functions;

import io.questdb.cairo.AbstractRecordCursorFactory;
import io.questdb.cairo.sql.RecordCursor;
import io.questdb.cairo.sql.RecordMetadata;
import io.questdb.griffin.SqlExecutionContext;

/**
 * This factory has limitations. In that it does not differentiate cursor.toTop() from creating new
 * cursor instance. Semantically toTop() does not refresh data snapshot and newInstance() does, or it supposed to.
 */
public class GenericRecordCursorFactory extends AbstractRecordCursorFactory {
    private final RecordCursor cursor;
    private final boolean supportsRandomAccess;

    public GenericRecordCursorFactory(RecordMetadata metadata, RecordCursor cursor, boolean supportsRandomAccess) {
        super(metadata);
        this.cursor = cursor;
        this.supportsRandomAccess = supportsRandomAccess;
    }

    @Override
    public RecordCursor getCursor(SqlExecutionContext executionContext) {
        cursor.toTop();
        return cursor;
    }

    @Override
    public boolean isRandomAccessCursor() {
        return supportsRandomAccess;
    }
}
